@@ -1,5 +1,5 @@
 from django import forms
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 VALIDATION_CHOICES = (
     ('is_valid', _('Observation is valid')),
