@@ -1,9 +1,9 @@
 (function($) {
-	$.fn.userautocomplete = function(url) {
+	$.fn.userautocomplete = function(options) {
 
 		var username_input = $( this );
 
-		var no_results_indicator = $("#id_search_user_no_results");
+		var no_results_indicator = $("#" + username_input.attr('id') +  "_no_results");
 
 		username_input.focusout(function(){
 			no_results_indicator.hide();
@@ -15,7 +15,7 @@
 
 				$.ajax({
 					type: "GET",
-					url: url,
+					url: options.url,
 					dataType: "json",
 					cache: false,
 					data: {
@@ -38,11 +38,9 @@
 			}, 
 			afterSelect : function(item){
 
-				$.get(item.edit_role_url, function(html){
-					$("#ModalContent").html(html);
-					ajaxify("ModalContent");
-				});
-
+				if (options.hasOwnProperty("afterSelect") && typeof(options.afterSelect) == 'function'){
+					options.afterSelect(item);
+				}
 			},
             autoSelect: false,
 			minLength: 3,
