@@ -6,10 +6,11 @@ from .lazy import LazyAppTaxon
 '''
     Model Mixins for Adding Taxa to models
     - loose link to any taxonomic source, defined by
-    -- taxon_uuid
-    -- taxon_nuid
     -- taxon_latname
+    -- taxon_author
     -- taxon_source
+    -- taxon_nuid
+    -- name_uuid
 '''
 class ModelWithTaxonCommon(models.Model):
 
@@ -28,12 +29,13 @@ class ModelWithTaxonCommon(models.Model):
             self.set_taxon(lazy_taxon)
 
         elif self.pk:
-            if self.taxon_uuid:
+
+            if self.taxon_latname and self.taxon_source and self.taxon_nuid and self.name_uuid:
                 lazy_taxon = self.LazyTaxonClass(instance=self)
                 self.set_taxon(lazy_taxon)
 
     def load_taxon(self):
-        if self.taxon_uuid:
+        if self.taxon_latname:
             taxon = self.get_taxon()
     
     '''
@@ -47,37 +49,60 @@ class ModelWithTaxonCommon(models.Model):
         else:
             # load the taxon instance from the source
 
-            if self.taxon_uuid and self.taxon_source:
-                TaxonTreeModel = self.get_source_model()
-                instance = TaxonTreeModel.objects.get(uuid=str(self.uuid)) 
-                lazy_taxon = LazyTaxon(instance)
+            if self.taxon_nuid and self.taxon_latname and self.taxon_source:
+
+                lazy_taxon_kwargs = {
+                    'name_uuid' : str(self.name_uuid),
+                    'taxon_nuid' : self.taxon_nuid,
+                    'taxon_latname' : self.taxon_latname,
+                    'taxon_author' : self.taxon_author,
+                    'taxon_source' : self.taxon_source,
+                    'taxon_include_descendants' : self.taxon_include_descendants,
+                }
+                lazy_taxon = LazyAppTaxon(**lazy_taxon_kwargs)
                 self.set_taxon(lazy_taxon)
                 
         return self.taxon
 
 
+    def set_taxon_parameters(self, lazy_taxon):
+
+        self.name_uuid = str(lazy_taxon.name_uuid)
+        self.taxon_nuid = lazy_taxon.taxon_nuid
+        self.taxon_latname = lazy_taxon.taxon_latname
+        self.taxon_author = lazy_taxon.taxon_author
+        self.taxon_source = lazy_taxon.taxon_source
+        self.taxon_include_descendants = lazy_taxon.taxon_include_descendants
+
+
     def set_taxon(self, lazy_taxon):
 
         # allow setting with LazyTaxon or LazyAppTaxon
         
-        self.taxon_uuid = str(lazy_taxon.taxon_uuid)
-        self.taxon_nuid = lazy_taxon.taxon_nuid
-        self.taxon_latname = lazy_taxon.taxon_latname
-        self.taxon_source = lazy_taxon.taxon_source
-        self.taxon_include_descendants = lazy_taxon.taxon_include_descendants
+        self.set_taxon_parameters(lazy_taxon)
 
         # the lazy taxon, make sure it is the right LazyTaxonClass
         lazy_taxon_kwargs = {
-            'taxon_uuid' : str(lazy_taxon.taxon_uuid),
-            'taxon_nuid' : lazy_taxon.taxon_nuid,
             'taxon_latname' : lazy_taxon.taxon_latname,
+            'taxon_author' : lazy_taxon.taxon_author,
             'taxon_source' : lazy_taxon.taxon_source,
             'taxon_include_descendants' : lazy_taxon.taxon_include_descendants,
+            'name_uuid' : str(lazy_taxon.name_uuid),
+            'taxon_nuid' : lazy_taxon.taxon_nuid,
         }
         
         self.taxon = self.LazyTaxonClass(**lazy_taxon_kwargs)
 
 
+    def remove_taxon(self):
+        self.name_uuid = None
+        self.taxon_nuid = None
+        self.taxon_latname = None
+        self.taxon_author = None
+        self.taxon_source = None
+        self.taxon_include_descendants = None
+
+
     def vernacular(self, language=None):
         if not self.taxon:
             self.get_taxon()
@@ -100,7 +125,7 @@ class ModelWithTaxonCommon(models.Model):
             vernacular = self.taxon.vernacular(language)
 
             if vernacular:
-                return '%s (%s)' %(vernacular, self.taxon_latname)
+                return '{0} ({1})'.format(vernacular, self.taxon_latname)
 
             return self.taxon_latname
             
@@ -108,12 +133,13 @@ class ModelWithTaxonCommon(models.Model):
 
 
     def save(self, *args, **kwargs):
+        
         if self.taxon is not None:
             self.set_taxon(self.taxon)
 
 
         # make sure there is no taxon saved without all parameters set correctly
-        taxon_required_fields = set(['taxon_source', 'taxon_nuid', 'taxon_latname', 'taxon_uuid'])
+        taxon_required_fields = set(['taxon_source', 'taxon_nuid', 'taxon_latname', 'name_uuid'])
 
         for field_name in taxon_required_fields:
 
@@ -130,7 +156,37 @@ class ModelWithTaxonCommon(models.Model):
                 break
 
         super().save(*args, **kwargs)
-        
+
+
+    '''
+    updating the taxon of ModelWithTaxon
+    '''
+    # only callable by LazyTaxon, not LazyAppTaxon
+    def update_taxon(self):
+        lazy_taxon = self.get_taxon()
+        if lazy_taxon:
+            if lazy_taxon.exists_in_tree():
+                tree_instance = lazy_taxon.tree_instance()
+
+                if str(tree_instance.name_uuid) != str(self.name_uuid) or tree_instance.taxon_nuid != self.taxon_nuid:
+
+                    new_lazy_taxon = LazyAppTaxon(instance=tree_instance)
+                    new_lazy_taxon.taxon_include_descendants = self.taxon_include_descendants
+                    
+                    self.set_taxon(new_lazy_taxon)
+                    self.save()
+
+            elif lazy_taxon.exists_as_synonym():
+
+                synonym_instance = lazy_taxon.synonym_instance()
+                tree_instance = synonym_instance.taxon
+                
+                new_lazy_taxon = LazyAppTaxon(instance=tree_instance)
+                new_lazy_taxon.taxon_include_descendants = self.taxon_include_descendants
+                    
+                self.set_taxon(new_lazy_taxon)
+                self.save()
+
         
     class Meta:
         abstract = True
@@ -144,20 +200,26 @@ class ModelWithTaxonCommon(models.Model):
 '''
 class ModelWithTaxon(ModelWithTaxonCommon):
 
-    taxon_uuid = models.UUIDField(null=True)
-    taxon_nuid = models.CharField(max_length=255, null=True)
     taxon_latname = models.CharField(max_length=255, null=True)
+    taxon_author = models.CharField(max_length=255, null=True)
     taxon_source = models.CharField(max_length=255, null=True)
 
     taxon_include_descendants = models.BooleanField(default=False)
 
+    taxon_nuid = models.CharField(max_length=255, null=True)
+    name_uuid = models.UUIDField(null=True)
+
+    
+
     def remove_taxon(self):
         self.taxon = None
-        self.taxon_uuid = None
-        self.taxon_nuid = None
         self.taxon_latname = None
+        self.taxon_author = None
         self.taxon_source = None
 
+        self.taxon_nuid = None
+        self.name_uuid = None
+
         self.save()
 
 
@@ -167,12 +229,14 @@ class ModelWithTaxon(ModelWithTaxonCommon):
         
 class ModelWithRequiredTaxon(ModelWithTaxonCommon):
     
-    taxon_uuid = models.UUIDField()
-    taxon_nuid = models.CharField(max_length=255)
     taxon_latname = models.CharField(max_length=255)
+    taxon_author = models.CharField(max_length=255, null=True) # some higher taxa have no author
     taxon_source = models.CharField(max_length=255)
 
     taxon_include_descendants = models.BooleanField(default=False)
 
+    taxon_nuid = models.CharField(max_length=255)
+    name_uuid = models.UUIDField()
+
     class Meta:
         abstract = True
