@@ -3,13 +3,18 @@ from django import forms
 from django.urls import reverse
 from django.utils.translation import gettext_lazy as _
 from django.contrib.contenttypes.models import ContentType
+from django.db.models import Q
 
-from .models import NAVIGATION_LINK_NAME_MAX_LENGTH, TemplateContent, Navigation, NavigationEntry, LocalizedTemplateContent
+from .models import (NAVIGATION_LINK_NAME_MAX_LENGTH, TemplateContent, Navigation, NavigationEntry,
+                     LocalizedTemplateContent, NavigationEntry, LocalizedNavigationEntry)
 
 from .Templates import Template, Templates
 
-from .fields import ComponentField
-from .widgets import ContentWithPreviewWidget, FileContentWidget, TextareaContentWidget, TextContentWidget
+from .fields import ComponentField, StreamField
+from .widgets import (ContentWithPreviewWidget, FileContentWidget, TextareaContentWidget, TextContentWidget,
+                      StreamContentWidget)
+
+from .utils import get_preview_text
 
 from django.contrib.auth import get_user_model
 
@@ -33,11 +38,10 @@ class CreateTemplateContentForm(TemplateContentFormCommon):
     
     template_name = forms.ChoiceField(label =_('Template'))
 
-    def __init__(self, app, template_type, *args, **kwargs):
+    def __init__(self, app, *args, **kwargs):
 
         self.app = app
-        self.template_type = template_type
-        self.assignment = kwargs.pop('assginment', None)
+        self.assignment = kwargs.pop('assignment', None)
         
         super().__init__(*args, **kwargs)
 
@@ -52,16 +56,16 @@ class CreateTemplateContentForm(TemplateContentFormCommon):
                 choice = (template_name, template.definition['templateName'])
                 choices.append(choice)
         self.fields['template_name'].choices = choices
+        
 
     def clean(self):
-        cleaned_data = super().clean()
-        assignment = cleaned_data.get('assignment')
 
-        if assignment:
-            assigned_template_content_exists = TemplateContent.objects.filter(app=self.app, assignment=assignment).exists()
+        if self.assignment:
+            assigned_template_content_exists = TemplateContent.objects.filter(app=self.app,
+                                                                              assignment=self.assignment).exists()
 
             if assigned_template_content_exists == True:
-                raise forms.ValidationError('A Template content for "{0}" already exists'.format(assignment))
+                raise forms.ValidationError('A template content for "{0}" already exists'.format(self.assignment))
 
 
 # translations initially do not supply a localized_template_content? - change this!!
@@ -76,24 +80,42 @@ class TemplateContentFormFieldManager:
         self.localized_template_content = localized_template_content
         self.primary_locale_template_content = template_content.get_locale(app.primary_language)
 
+    # allowMultiple=False
+    def get_instance(self, content_key, content_type):
+        
+        instance = None
+        
+        if self.localized_template_content:
+            if content_type == 'image':
+                image_type = self._get_image_type(content_key)
+                instance = self.localized_template_content.image(image_type=image_type)
 
+            elif content_type in ['component', 'text', 'templateContentLink'] and self.localized_template_content.draft_contents:
+                instance = self.localized_template_content.draft_contents.get(content_key, None)
+        
+        return instance
+    
+    # allowMultiple=True
     def get_instances(self, content_key, content_type):
 
         instances = []
 
-        if content_type == 'image':
-            image_type = self._get_image_type(content_key)
-            print('.images')
-            instances = list(self.localized_template_content.images(image_type=image_type).order_by('pk'))
+        if self.localized_template_content:
+            if content_type == 'image':
+                image_type = self._get_image_type(content_key)
+                instances = list(self.localized_template_content.images(image_type=image_type).order_by('pk'))
 
-        elif content_type in ['component', 'text', 'templateContentLink'] and self.localized_template_content.draft_contents:
-            instances = self.localized_template_content.draft_contents.get(content_key, [])
+            elif content_type in ['component', 'text', 'templateContentLink', 'stream'] and self.localized_template_content.draft_contents:
+                instances = self.localized_template_content.draft_contents.get(content_key, [])
 
         return instances
 
 
     def get_primary_locale_content(self, content_key):
-        self.primary_locale_template_content.draft_contents.get(content_key, 'None')
+        content = None
+        if self.primary_locale_template_content.draft_contents:
+            content = self.primary_locale_template_content.draft_contents.get(content_key, None)
+        return content
 
 
     def _add_primary_locale_content_to_form_field(self, form_field, content_key):
@@ -106,20 +128,35 @@ class TemplateContentFormFieldManager:
         return form_field
 
 
-
+    '''
+        {
+            "title": "Sample page",
+            "templateName": "Sample",
+            "templatePath": "/template_content/page/sample/sample.html",
+            "version": 1,
+            "contents": {
+                "stream": [
+                    {
+                        "templateName": "Video",
+                        "type": "component",
+                        (...other component fields...)
+                    }
+                ]
+            }
+        }
+    '''
     def get_form_fields(self, content_key, content_definition):
-
-        instances = self.get_instances(content_key, content_definition['type'])
-
+        
         form_fields = []
-
+        
         content_type = content_definition['type']
         allow_multiple = content_definition.get('allowMultiple', False)
 
         field_getter_name = '_get_{0}_form_field'.format(content_type)
         field_getter = getattr(self, field_getter_name)
 
-        if allow_multiple == True:
+        if content_type != 'stream' and allow_multiple == True:
+            instances = self.get_instances(content_key, content_definition['type'])
             max_number = content_definition.get('maxNumber', None)
 
             is_first = True
@@ -171,8 +208,9 @@ class TemplateContentFormFieldManager:
 
         else:
             
-            instance = instances
+            instance = self.get_instance(content_key, content_definition['type'])
 
+            # how could it be a list?
             if isinstance(instance, list) and len(instance) > 0:
                 instance = instance[0]
 
@@ -245,12 +283,14 @@ class TemplateContentFormFieldManager:
         if current_image:
 
             data_url_kwargs = {
+                'app_uid' : self.app.uid,
                 'content_image_id' : current_image.id,
             }
 
             data_url = reverse(self.manage_content_image_url_name, kwargs=data_url_kwargs)
             
             delete_kwargs = {
+                'app_uid': self.app.uid,
                 'pk' : current_image.pk,
             }
 
@@ -262,6 +302,7 @@ class TemplateContentFormFieldManager:
                 ltc_content_type = ContentType.objects.get_for_model(self.localized_template_content)
 
                 data_url_kwargs = {
+                    'app_uid' : self.app.uid,
                     'content_type_id' : ltc_content_type.id,
                     'object_id' : self.localized_template_content.id,
                     'image_type' : image_type
@@ -270,22 +311,41 @@ class TemplateContentFormFieldManager:
                 data_url = reverse(self.manage_content_image_url_name, kwargs=data_url_kwargs)
             
         widget_attrs['data_url'] = data_url
-        widget_attrs['delete_url'] = delete_url  
+        widget_attrs['delete_url'] = delete_url
+        widget_attrs['accept'] = 'image/png, image/webp, image/jpeg'
 
         widget = FileContentWidget(widget_attrs)
         form_field = forms.ImageField(widget=widget, **field_kwargs)
 
-        return form_field
+        # add primary locale ServerContentImage as form_field.primary_locale_content
+        allow_multiple = content_definition.get('allowMultiple', False)
+        if allow_multiple == True:
+            primary_locale_images = list(self.primary_locale_template_content.images(image_type=content_key).order_by('pk'))
+        else:
+            primary_locale_images = self.primary_locale_template_content.images(image_type=content_key).order_by('pk').last()
 
+        form_field.primary_locale_content = primary_locale_images
+
+        return form_field
 
     def _get_text_form_field(self, content_key, content_definition, instance=None):
 
         field_kwargs = self._get_common_field_kwargs(content_key, content_definition)
 
+        widget_name = content_definition.get('widget', None)
+        
         widget = TextareaContentWidget
-        if content_definition.get('widget', None) == 'TextInput':
+        field_class = forms.CharField
+        
+        if widget_name == 'TextInput':
             widget = TextContentWidget
-
+        elif widget_name == 'Select':
+            choices_definition = content_definition.get('choices', [])
+            choices = [(option, option) for option in choices_definition]
+            field_kwargs['choices'] = choices
+            widget = forms.Select
+            field_class = forms.ChoiceField
+            
         initial = ''
         if instance:
             initial = instance
@@ -295,7 +355,7 @@ class TemplateContentFormFieldManager:
             'initial' : initial,
         })
                                             
-        form_field = forms.CharField(**field_kwargs)
+        form_field = field_class(**field_kwargs)
 
         form_field = self._add_primary_locale_content_to_form_field(form_field, content_key)
         
@@ -323,14 +383,12 @@ class TemplateContentFormFieldManager:
             data_url_kwargs['component_uuid'] = instance['uuid']
             delete_url = reverse('delete_component', kwargs=data_url_kwargs)
 
-            if 'identifierContent' in component_template.definition:
-                identifier_key = component_template.definition['identifierContent']
-                preview_text = instance.get(identifier_key, None)
+            preview_text = get_preview_text(component_template, instance)
 
-                if isinstance(preview_text, dict):
-                    preview_text = preview_text.get('title', None)
-
-        data_url = reverse('manage_component', kwargs=data_url_kwargs)
+        if instance:
+            data_url = reverse('manage_component', kwargs=data_url_kwargs)
+        else:
+            data_url = reverse('add_component', kwargs=data_url_kwargs)
 
         widget_attrs['data_url'] = data_url
         widget_attrs['delete_url'] = delete_url
@@ -350,6 +408,21 @@ class TemplateContentFormFieldManager:
         form_field = self._add_primary_locale_content_to_form_field(form_field, content_key)
 
         return form_field
+    
+    
+    def _get_stream_form_field(self, content_key, content_definition, instance=None):
+        
+        field_kwargs = self._get_common_field_kwargs(content_key, content_definition)
+        
+        widget = StreamContentWidget(self.app, self.localized_template_content, content_key, content_definition)
+        
+        field_kwargs.update({
+            'widget' : widget,
+        })
+        
+        form_field = StreamField(**field_kwargs)
+        
+        return form_field
 
 
     def _get_templateContentLink_form_field(self, content_key, content_definition, instance=None):
@@ -386,7 +459,9 @@ class ComponentFormFieldManager(TemplateContentFormFieldManager):
     manage_content_image_url_name = 'manage_component_image'
     delete_content_image_url_name = 'delete_component_image'
 
+    # component_uuid can be None for new components
     def __init__(self, app, template_content, localized_template_content, component_key, component_uuid, component={}):
+        self.app = app
         self.template_content = template_content
         self.localized_template_content = localized_template_content
         self.primary_locale_template_content = template_content.get_locale(app.primary_language)
@@ -399,6 +474,22 @@ class ComponentFormFieldManager(TemplateContentFormFieldManager):
         return get_component_image_type(self.component_key, self.component_uuid, content_key)
 
 
+    def get_instance(self, content_key, content_type):
+        
+        instance = None
+
+        if self.component:
+            
+            if content_type == 'image':
+                image_type = self._get_image_type(content_key)
+                instance = self.localized_template_content.image(image_type=image_type)
+
+            elif content_type in ['component', 'text', 'templateContentLink']:
+                instance = self.component.get(content_key, None)
+
+        return instance
+    
+        
     def get_instances(self, content_key, content_type):
 
         instances = []
@@ -426,12 +517,12 @@ class ManageLocalizedTemplateContentForm(TemplateContentFormCommon):
 
     def __init__(self, app, template_content, localized_template_content=None, *args, **kwargs):
 
+        super().__init__(*args, **kwargs)
+        
         self.language = kwargs.get('language', None)
         if localized_template_content:
             self.language = localized_template_content.language
 
-        super().__init__(*args, **kwargs)
-
         self.app = app
         self.template_content = template_content
         self.localized_template_content = localized_template_content
@@ -440,8 +531,9 @@ class ManageLocalizedTemplateContentForm(TemplateContentFormCommon):
         self.layoutable_simple_fields = set([])
 
         self.set_template_definition()
-        self.set_form_fields()
 
+        if self.localized_template_content:
+            self.set_form_fields()
 
     def set_template_definition(self):
         self.template_definition = self.template_content.draft_template.definition
@@ -461,19 +553,21 @@ class ManageLocalizedTemplateContentForm(TemplateContentFormCommon):
             form_fields = field_manager.get_form_fields(content_key, content_definition)
 
             # get form fields for each content_id
-            for field in form_fields:
-
-                field['field'].content_definition = content_definition
-                field['field'].content_key = content_key
+            for field_definition in form_fields:
                 
-                self.fields[field['name']] = field['field']
+                field = field_definition['field']
+                field_name = field_definition['name']
 
-                self.fields[field['name']].language = self.language
+                field.content_definition = content_definition
+                field.content_key = content_key
+                field.language = self.language
+                
+                self.fields[field_name] = field
                 
                 if content_definition.get('format', None) == 'layoutable-simple':
-                    self.layoutable_simple_fields.add(field['name'])
+                    self.layoutable_simple_fields.add(field_name)
                 elif content_definition.get('format', None) == 'layoutable-full':
-                    self.layoutable_full_fields.add(field['name'])
+                    self.layoutable_full_fields.add(field_name)
 
 
 class ManageComponentForm(ManageLocalizedTemplateContentForm):
@@ -482,28 +576,36 @@ class ManageComponentForm(ManageLocalizedTemplateContentForm):
 
     localizeable_fields = []
 
-    def __init__(self, app, template_content, localized_template_content, content_key, component=None, *args, **kwargs):
+    def __init__(self, app, template_content, localized_template_content, content_key, component_template_name, component=None, *args, **kwargs):
+        initial = kwargs.get('initial', {})
+        
+        if 'uuid' not in initial:
+            raise ValueError("initial['uuid] is required")
+
+        self.component_template_name = component_template_name
+
+
         self.content_key = content_key
         self.component = component
         
-        kwargs['auto_id'] = 'id_{0}_%s'.format(content_key)
         super().__init__(app, template_content, localized_template_content, *args, **kwargs)
 
         self.fields.pop('draft_title')
-
+        
     def set_template_definition(self):
-        page_template_definition = self.template_content.draft_template.definition
-        self.component_template_name = page_template_definition['contents'][self.content_key]['templateName']
-
         # load the component template
         component_template = Template(self.app, self.component_template_name, 'component')
-
         self.template_definition = component_template.definition
 
     def get_form_field_manager(self):
         component_uuid = self.initial['uuid']
         return ComponentFormFieldManager(self.app, self.template_content, self.localized_template_content, self.content_key,
                                         component_uuid, self.component)
+        
+    def add_prefix(self, field_name):
+        # look up field name; return original if not found
+        field_name = '{0}_{1}'.format(self.content_key, field_name)
+        return super().add_prefix(field_name)
 
 
 
@@ -524,13 +626,16 @@ class ManageNavigationForm(LocalizeableForm):
         super().__init__(*args, **kwargs)
         # read navigation_type choices from frontend
         frontend_settings = app.get_settings()
-        navigations = frontend_settings['templateContent']['navigations']
+        navigations = {}
+        
+        if 'templateContent' in frontend_settings and 'navigations' in frontend_settings['templateContent']:
+            navigations = frontend_settings['templateContent']['navigations']
 
         choices = []
 
         for navigation_type, definition in navigations.items():
 
-            if settings.LOCALCOSMOS_PRIVATE == True and definition['offline'] == True:
+            if settings.LOCALCOSMOS_PRIVATE == True and definition.get('offline', False) == True:
                 continue
 
             choice = (navigation_type, definition['name'])
@@ -574,11 +679,14 @@ class ManageNavigationEntryForm(LocalizeableForm):
 
         if self.max_levels == 1:
             parent_queryset = NavigationEntry.objects.none()
-
+            
         elif self.max_levels == 2:
             parent_queryset = NavigationEntry.objects.filter(navigation=navigation, parent=None)
+            
         else:
-            parent_queryset = NavigationEntry.objects.filter(navigation=navigation, parent__parent=None)
+            parent_q = Q(parent=None)
+            grandparent_q = Q(parent__parent=None)
+            parent_queryset = NavigationEntry.objects.filter(Q(navigation=navigation), parent_q | grandparent_q)
 
         if self.navigation_entry:
             exclude_pks = [self.navigation_entry.pk]
@@ -596,6 +704,52 @@ class ManageNavigationEntryForm(LocalizeableForm):
 
 
 
+class TranslateNavigationForm(LocalizeableForm):
+
+    name = forms.CharField(max_length=355, label=_('Name of the navigation'))
+    localizeable_fields = ['name']
+    
+    def __init__(self, app, navigation, *args, **kwargs):
+        self.navigation = navigation
+        self.primary_language = app.primary_language
+        self.language = kwargs['language']
+        self.primary_locale_navigation = navigation.get_locale(self.primary_language)
+        self.navigation_entries = NavigationEntry.objects.filter(navigation=self.navigation)
+
+        self.localized_navigation = navigation.get_locale(self.language)
+        super().__init__(*args, **kwargs)
+
+        self.set_form_fields()
 
 
+    def set_form_fields(self):
+        
+        self.fields['name'].primary_locale_text = self.primary_locale_navigation.name
 
+        for navigation_entry in self.navigation_entries:
+
+            primary_locale_navigation_entry = navigation_entry.get_locale(self.primary_language) 
+            localized_navigation_entry = navigation_entry.get_locale(self.language)
+            widget = forms.TextInput
+
+            initial = ''
+            if localized_navigation_entry:
+                initial = localized_navigation_entry.link_name
+
+            label = '{0}'.format(primary_locale_navigation_entry.link_name)
+                                                
+            field_kwargs = {
+                'widget' : widget,
+                'initial' : initial,
+                'label': label,
+                'required': False
+            }
+                                                
+            form_field = forms.CharField(**field_kwargs)
+            form_field.primary_locale_text = primary_locale_navigation_entry.link_name
+            form_field.language = self.language
+            form_field.navigation_entry = navigation_entry
+
+            field_name = 'ne-{0}'.format(navigation_entry.id)
+
+            self.fields[field_name] = form_field
\ No newline at end of file
