@@ -3,6 +3,9 @@ from django.conf import settings
 from django.utils.translation import gettext_lazy as _
 from django.template.defaultfilters import slugify
 from django.core.files import File
+from django.contrib.contenttypes.fields import GenericRelation
+
+from localcosmos_server.taxonomy.generic import ModelWithTaxon
 
 from localcosmos_server.models import App, ServerContentImageMixin, TaxonomicRestriction
 
@@ -10,7 +13,7 @@ from django.utils import timezone
 
 import os, json, uuid, shutil
 
-from.Templates import Template
+from .Templates import Template
 
 from .utils import (get_component_image_type, get_published_image_type, PUBLISHED_IMAGE_TYPE_PREFIX,
                     get_frontend_specific_url)
@@ -20,9 +23,18 @@ TEMPLATE_TYPES = (
     ('component', _('Component')),
 )
 
+TEMPLATE_CONTENT_TYPES = (
+    'text',
+    'image',
+    'component',
+    'stream',
+    'templateContentLink',
+)
+
 NAVIGATION_LINK_NAME_MAX_LENGTH = 20
 TITLE_MAX_LENGTH = 255
 
+# a relative path
 def get_published_template_content_root(template_content):
 
     template_definition = template_content.draft_template.definition
@@ -148,7 +160,7 @@ class TemplateContentManager(models.Manager):
         return template_content
 
     
-    def filter_by_taxon(self, lazy_taxon, ascendants=False):
+    def filter_by_taxon(self, app, lazy_taxon, ascendants=False):
 
         template_contents = []
 
@@ -158,16 +170,15 @@ class TemplateContentManager(models.Manager):
 
             template_content_ids = template_content_links.values_list('object_id', flat=True)
 
-            template_contents = self.filter(pk__in=template_content_ids)
+            template_contents = self.filter(app=app, pk__in=template_content_ids)
 
 
         else:
-            # get for all nuids, not implemented yet
             template_content_links = TaxonomicRestriction.objects.get_for_taxon_branch(TemplateContent, lazy_taxon)
 
             template_content_ids = template_content_links.values_list('object_id', flat=True)
 
-            template_contents = self.filter(pk__in=template_content_ids)
+            template_contents = self.filter(app=app, pk__in=template_content_ids)
 
         
         return template_contents
@@ -179,8 +190,8 @@ class TemplateContentManager(models.Manager):
     - template content which is not available offline can be fetched using the API
     - this can only work if some sort of menu is being fetched from the server
     - template content does not have to be part of a menu
-    - offline (build) template content is never newer than the online version
-    - the app can query online to get new contents, should also fetch template then
+    - offline (built) template content is never newer than the online version
+    - the app can query api to get new contents, should also fetch template (+definition) then
 
     published_template:
         The actual template and its definition are stored in the database rather then store a path 
@@ -206,6 +217,8 @@ class TemplateContent(PublicationMixin, models.Model):
 
     published_template = models.FileField(null=True, upload_to=get_published_page_template_path)
     published_template_definition = models.FileField(null=True, upload_to=get_published_page_template_definition_path)
+    
+    taxonomic_restrictions = GenericRelation(TaxonomicRestriction)
 
     objects = TemplateContentManager()
 
@@ -214,25 +227,33 @@ class TemplateContent(PublicationMixin, models.Model):
         # use the templates on disk
         self.draft_template = Template(self.app, self.draft_template_name, self.template_type)
 
+    @property
+    def template(self):
+        template = None
+        
         if self.published_template:
             # the published template, use the template data (definition&template) stored in the db
-            self.template = Template(self.app, self.name, self.template_type,
+            template = Template(self.app, self.name, self.template_type,
                 self.published_template.path, self.published_template_definition.path)
-
+            
+        return template
 
     # return different path for published pages
-    def get_component_template(self, content_key):
-
-        component_template_name = self.draft_template.definition['contents'][content_key]['templateName']
+    # stream items will provide component_template_name
+    # content_key is not suffucient for stream items, because different components share the same content_key)
+    def get_component_template(self, content_key, component_template_name=None):
+        
+        if not component_template_name:
+            component_template_name = self.draft_template.definition['contents'][content_key]['templateName']
         # load the component template
         component_template = Template(self.app, component_template_name, 'component')
 
         return component_template
 
 
-    def get_published_component_template(self, content_key):
-
-        component_template_name = self.template.definition['contents'][content_key]['templateName']
+    def get_published_component_template(self, content_key, component_template_name=None):
+        if not component_template_name:
+            component_template_name = self.template.definition['contents'][content_key]['templateName']
         
         published_component_template_definition_filepath = self.get_published_component_template_definition_filepath(component_template_name)
         published_component_template_filepath = self.get_published_component_template_filepath(component_template_name)
@@ -266,14 +287,18 @@ class TemplateContent(PublicationMixin, models.Model):
 
     @property
     def name (self):
+        if not self.draft_template.template_exists:
+            return self.draft_template_name
+        
         return self.draft_template.definition['templateName']
-
+        
     def get_locale(self, language_code):
         return LocalizedTemplateContent.objects.filter(template_content=self, language=language_code).first()
 
 
     def publish_assets(self):
 
+        # relative to settings.MEDIA_ROOT as required by django
         published_templates_root = get_published_template_content_root(self)
 
         if self.published_template.name and self.published_template.storage.exists(self.published_template.name):
@@ -282,14 +307,15 @@ class TemplateContent(PublicationMixin, models.Model):
         if self.published_template_definition.name and self.published_template_definition.storage.exists(self.published_template_definition.name):
             self.published_template_definition.storage.delete(self.published_template_definition.name)
 
-        if os.path.isdir(published_templates_root):
-            shutil.rmtree(published_templates_root)
+        absolute_published_templates_root = os.path.join(settings.MEDIA_ROOT, published_templates_root)
+        if os.path.isdir(absolute_published_templates_root):
+            shutil.rmtree(absolute_published_templates_root)
 
         # store TemplateContent.published_template and TemplateContent.published_template_definition
         filepaths = [self.draft_template.template_filepath, self.draft_template.template_definition_filepath]
 
         for filepath in filepaths:
-
+            
             with open(filepath, 'r') as template_file:
                 filename = os.path.basename(filepath)
                 djangofile = File(template_file)
@@ -303,16 +329,30 @@ class TemplateContent(PublicationMixin, models.Model):
         # store components templates
         template_definition = self.draft_template.definition
 
-        published_templates_root = get_published_template_content_root(self)
-        published_component_templates_root = os.path.join(published_templates_root, 'templates', 'component')
-
+        # published_templates_root = get_published_template_content_root(self)
+        # published_component_templates_root = os.path.join(published_templates_root, 'templates', 'component')
+        copied_component_template_names = []
+        
         for content_key, content_definition in template_definition['contents'].items():
+            
+            component_template_names = []
 
             if content_definition['type'] == 'component':
-
                 component_template_name = content_definition['templateName']
+            
+            elif content_definition['type'] == 'stream':
+                component_template_names += content_definition['allowedComponents']
+                
+            component_template_names = set(component_template_names)
 
-                component_template = self.get_component_template(content_key)
+            for component_template_name in component_template_names:
+
+                if component_template_name in copied_component_template_names:
+                    continue
+
+                copied_component_template_names.append(component_template_name)
+
+                component_template = self.get_component_template(content_key, component_template_name)
 
                 # does not exist
                 published_component_template_folder = self.get_published_component_template_folder(
@@ -329,7 +369,6 @@ class TemplateContent(PublicationMixin, models.Model):
                 published_component_template_definition_filepath = os.path.join(published_component_template_folder,
                     component_template_definition_filename)
 
-                print(published_component_template_definition_filepath)
                 shutil.copyfile(component_template.template_filepath, published_component_template_filepath)
                 shutil.copyfile(component_template.template_definition_filepath,
                     published_component_template_definition_filepath)
@@ -360,6 +399,7 @@ class TemplateContent(PublicationMixin, models.Model):
 
 
     class Meta:
+        # this constraint might be wrong. There could be more than one page with app/None assignment
         unique_together=('app', 'assignment')
 
 
@@ -459,8 +499,11 @@ class LocalizedTemplateContent(ServerContentImageMixin, models.Model):
 
                     if 'required' in content_definition and content_definition['required'] == False:
                         continue
-
-                    content = self.draft_contents.get(content_key, None)
+                    
+                    content = None
+                    
+                    if self.draft_contents:
+                        content = self.draft_contents.get(content_key, None)
 
                     if not content:
                         translation_errors.append(_('The component "%(component_name)s" is required but still missing for the language %(language)s.') %{'component_name':content_key, 'language':self.language})
@@ -475,10 +518,16 @@ class LocalizedTemplateContent(ServerContentImageMixin, models.Model):
                 translation_errors.append(_('Content is still missing for the language %(language)s.') % {'language':primary_language})
 
             else:
-                for content_key, content in primary_contents.items():
-                    if content_key not in self.draft_contents or not self.draft_contents[content_key]:
-                        translation_errors.append(_('The translation for the language %(language)s is incomplete.') % {'language':self.language})
-                        break
+                
+                error_message = _('The translation for the language %(language)s is incomplete.') % {'language':self.language}
+                if self.draft_contents:
+                    for content_key, content in primary_contents.items():
+                        if content_key not in self.draft_contents or not self.draft_contents[content_key]:
+                            translation_errors.append(error_message)
+                            break
+                        
+                else:
+                    translation_errors.append(error_message)
 
         return translation_errors
 
@@ -522,6 +571,24 @@ class LocalizedTemplateContent(ServerContentImageMixin, models.Model):
                             image_type = get_component_image_type(component_key, component_uuid, content_key)
 
                             self.publish_images(image_type, content_definition)
+                            
+                            
+            elif component_definition['type'] == 'stream':
+
+                stream_items = self.draft_contents.get(component_key, [])
+                for stream_item in stream_items:
+                    
+                    stream_item_uuid = stream_item['uuid']
+                    component_template_name = stream_item['templateName']
+                    component_template = self.template_content.get_component_template(component_key, component_template_name)
+                    
+                    for content_key, content_definition in component_template.definition['contents'].items():
+                        
+                        # publish the image and add the url to the component
+                        if content_definition['type'] == 'image':
+                            image_type = get_component_image_type(component_key, stream_item_uuid, content_key)
+
+                            self.publish_images(image_type, content_definition)
 
 
     def publish_images(self, image_type, content_definition):
@@ -580,7 +647,7 @@ class LocalizedTemplateContent(ServerContentImageMixin, models.Model):
 
         self.save(published=True)
 
-
+    # JSON SCHEMA VALIDATION NEEDED!
     def save(self, *args, **kwargs):
 
         # indicates, if the save() command came from self.publish
@@ -612,37 +679,49 @@ class LocalizedTemplateContent(ServerContentImageMixin, models.Model):
 
     def get_component(self, component_key, component_uuid):
 
-        allow_multiple = self.template_content.draft_template.definition['contents'][component_key].get(
-            'allowMultiple', False)
-
         component = {}
+        
+        template_definition = self.template_content.draft_template.definition
+        
+        if component_key in template_definition['contents']:
+            
+            allow_multiple = template_definition['contents'][component_key].get(
+                'allowMultiple', False)
+            
+            # check for type "stream", which is always mutliple
+            if template_definition['contents'][component_key]['type'] == 'stream':
+                allow_multiple = True
 
-        if self.draft_contents:
+            if self.draft_contents:
 
-            if allow_multiple == True:
-                
-                components = self.draft_contents.get(component_key, [])
+                if allow_multiple == True:
+                    
+                    components = self.draft_contents.get(component_key, [])
 
-                for possible_component in components:
+                    for possible_component in components:
 
-                    if str(possible_component['uuid']) == str(component_uuid):
-                        component = possible_component
-                        break
+                        if str(possible_component['uuid']) == str(component_uuid):
+                            component = possible_component
+                            break
 
-            else:
-                component = self.draft_contents.get(component_key, {})
+                else:
+                    component = self.draft_contents.get(component_key, {})
 
         return component
 
-
     def add_or_update_component(self, component_key, component, save=True):
 
         if 'uuid' not in component or not component['uuid']:
             raise ValueError('Cannot add component without uuid')
+        
+        content_definition = self.template_content.draft_template.definition['contents'][component_key]
 
-        allow_multiple = self.template_content.draft_template.definition['contents'][component_key].get(
+        allow_multiple = content_definition.get(
             'allowMultiple', False)
 
+        if content_definition['type'] == 'stream':
+            allow_multiple = True
+
         if not self.draft_contents:
             self.draft_contents = {}
 
@@ -670,9 +749,14 @@ class LocalizedTemplateContent(ServerContentImageMixin, models.Model):
 
 
     def remove_component(self, component_key, component_uuid, save=True):
+        
+        content_definition = self.template_content.draft_template.definition['contents'][component_key]
 
-        allow_multiple = self.template_content.draft_template.definition['contents'][component_key].get(
+        allow_multiple = content_definition.get(
             'allowMultiple', False)
+        
+        if content_definition['type'] == 'stream':
+            allow_multiple = True
 
         if not self.draft_contents:
             self.draft_contents = {}
@@ -866,13 +950,30 @@ class LocalizedNavigation(models.Model):
 
 
     def translation_complete(self):
+        translation_errors = []
         
         primary_language = self.navigation.app.primary_language
 
-        if self.language == primary_language:
-            return []
+        if self.language != primary_language:
 
-        raise NotImplementedError('multilanguage navigations are not implemented yet')
+            navigation_entries = NavigationEntry.objects.filter(navigation=self.navigation)
+
+            for navigation_entry in navigation_entries:
+                localized_navigation_entry = navigation_entry.get_locale(self.language)
+
+                if not localized_navigation_entry:
+                    translation_errors.append(_('The translation for the language %(language)s is incomplete.') % {'language':self.language})
+                    break
+
+        return translation_errors
+
+
+    @property
+    def translation_is_complete(self):
+        translation_errors = self.translation_complete()
+        if translation_errors:
+            return False
+        return True
 
 
     def publish(self):
@@ -881,7 +982,6 @@ class LocalizedNavigation(models.Model):
 
         if self.published_version != self.draft_version:
             self.published_version = self.draft_version
-            self.published_at = timezone.now()
 
         self.save(published=True)
 
@@ -984,7 +1084,8 @@ class LocalizedNavigationEntry(models.Model):
             url = self.url
 
         serialized_entry = {
-            'link_name': self.link_name,
+            'linkName': self.link_name,
+            'link_name': self.link_name, # backwards compatibility, deprecate it in the future
             'url': url,
             'children': [],
         }
