@@ -1,25 +1,30 @@
 from django.shortcuts import redirect
 from django.conf import settings
 from django.views.generic import TemplateView, FormView
+from django.http import JsonResponse
 from django import forms
 
 from localcosmos_server.models import App
 from localcosmos_server.generic_views import AjaxDeleteView
-from localcosmos_server.views import ManageServerContentImage, DeleteServerContentImage
+from localcosmos_server.views import ManageServerContentImageWithText, DeleteServerContentImage
 from localcosmos_server.view_mixins import AppMixin, FormLanguageMixin
 
 from localcosmos_server.decorators import ajax_required
 from django.utils.decorators import method_decorator
 
-from .models import TemplateContent, LocalizedTemplateContent, Navigation, NavigationEntry, LocalizedNavigationEntry
+from .models import (TemplateContent, LocalizedTemplateContent, Navigation, LocalizedNavigation,
+                     NavigationEntry, LocalizedNavigationEntry)
 from .forms import (CreateTemplateContentForm, ManageLocalizedTemplateContentForm, TranslateTemplateContentForm,
-                    ManageNavigationForm, ManageNavigationEntryForm, ManageComponentForm)
+                    ManageNavigationForm, ManageNavigationEntryForm, ManageComponentForm, TranslateNavigationForm,
+                    TemplateContentFormFieldManager)
 
 from .utils import get_frontend_specific_url
 
+from .Templates import Template
+
 from urllib.parse import urljoin
 
-import uuid
+import uuid, json
 
 
 class TemplateContentList(AppMixin, TemplateView):
@@ -28,36 +33,63 @@ class TemplateContentList(AppMixin, TemplateView):
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
+
+        # check if the frontend supports navigations
+        frontend_settings = self.app.get_settings()
+        supports_navigations = False
+
+        if 'templateContent' in frontend_settings and 'navigations' in frontend_settings['templateContent']:
+            supports_navigations = True
+
+        context['supports_navigations'] = supports_navigations
         
         localized_template_contents = LocalizedTemplateContent.objects.filter(template_content__app=self.app,
             template_content__template_type='page', language=self.app.primary_language, template_content__assignment=None).order_by('pk')
+        
+        found_template_content_ids = list(localized_template_contents.values_list('template_content__pk', flat=True))
+        
         context['localized_template_contents'] = localized_template_contents
 
         navigations = Navigation.objects.filter(app=self.app)
         context['navigations'] = navigations
 
         required_offline_contents = []
+        
+        # offline contents are always manged within the app kit
         if settings.LOCALCOSMOS_PRIVATE == False:
+                        
             app_settings = self.app.get_settings()
+            
+            if 'templateContent' in app_settings:
 
-            pages = app_settings['templateContent'].get('requiredOfflineContents', {})
-            for assignment, definition in pages.items():
+                required_contents = app_settings['templateContent'].get('requiredOfflineContents', {})
+                for assignment, definition in required_contents.items():
 
-                template_type = definition['templateType']
+                    template_type = definition['templateType']
 
-                template_content = TemplateContent.objects.filter(app=self.app, template_type=template_type,
-                    assignment=assignment).first()
+                    template_content = TemplateContent.objects.filter(app=self.app, template_type=template_type,
+                        assignment=assignment).first()
+                    
+                    if template_content:
+                        found_template_content_ids.append(template_content.pk)
 
-                content = {
-                    'assignment': assignment,
-                    'template_content': template_content,
-                    'template_type': template_type,
-                }
-
-                required_offline_contents.append(content)
+                    content = {
+                        'assignment': assignment,
+                        'template_content': template_content,
+                        'template_type': template_type,
+                    }
 
+                    required_offline_contents.append(content)
+            
+        
         context['required_offline_contents'] = required_offline_contents
 
+        # unsupported template contents
+        unsupported_contents = LocalizedTemplateContent.objects.filter(template_content__app=self.app, template_content__template_type='page',
+            language=self.app.primary_language).exclude(template_content__pk__in=found_template_content_ids).order_by('pk')
+            
+        context['unsupported_contents'] = unsupported_contents
+
         return context
 
 
@@ -87,10 +119,9 @@ class CreateTemplateContent(AppMixin, FormView):
 
 
     def get_form(self, form_class=None):
-        """Return an instance of the form to be used in this view."""
         if form_class is None:
             form_class = self.get_form_class()
-        return form_class(self.app, self.kwargs['template_type'], **self.get_form_kwargs())
+        return form_class(self.app, **self.get_form_kwargs())
 
 
     def form_valid(self, form):
@@ -164,7 +195,7 @@ class ManageTemplateContentCommon:
 
         initial = {}
         
-        if self.save_localized_template_content:
+        if self.localized_template_content:
             initial = {
                 'draft_title' : self.localized_template_content.draft_title,
                 'input_language' : self.localized_template_content.language,
@@ -186,7 +217,7 @@ class ManageTemplateContentCommon:
 
         for content_key, content_definition in template_definition['contents'].items():
 
-            if content_definition['type'] in ['image', 'component']:
+            if content_definition['type'] in ['image', 'component', 'stream']:
                 if content_key in old_keys:
                     old_keys.remove(content_key)
 
@@ -257,6 +288,17 @@ class ManageLocalizedTemplateContent(ManageTemplateContentCommon, AppMixin, With
     def dispatch(self, request, *args, **kwargs):
         self.set_template_content(**kwargs)    
         return super().dispatch(request, *args, **kwargs)
+    
+    def get_in_app_url(self):
+        app_settings = self.app.get_settings()
+        url_pattern = app_settings['templateContent']['urlPattern']
+        url = url_pattern.replace('{slug}', self.localized_template_content.slug).replace('{templateName}', self.template_content.draft_template_name)
+        return url
+    
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context['url_in_app'] = self.get_in_app_url()
+        return context
 
     def form_valid(self, form):
         
@@ -277,18 +319,8 @@ class ManageComponent(ManageTemplateContentCommon, AppMixin, WithLocalizedTempla
         self.set_template_content(**kwargs)
         self.set_component(**kwargs)
         return super().dispatch(request, *args, **kwargs)
-
-
-    def get_initial(self):
-        initial = super().get_initial()
-        if self.component:
-            initial['uuid'] = self.component['uuid']
-        else:
-            initial['uuid'] = uuid.uuid4()
-
-        return initial
-
-
+    
+    
     def set_component(self, **kwargs):
         self.app = App.objects.get(uid=kwargs['app_uid'])
         self.component_uuid = kwargs.pop('component_uuid', None)
@@ -296,15 +328,41 @@ class ManageComponent(ManageTemplateContentCommon, AppMixin, WithLocalizedTempla
         
         # load the component template
         self.component = {}
+        self.component_template_name = None
 
         if self.component_uuid:
             self.component = self.localized_template_content.get_component(self.content_key, self.component_uuid)
+            self.component_template_name = self.component.get('templateName', None)
+        
+        
+        if not self.component_template_name:
+            if 'component_template_name' in kwargs:
+                self.component_template_name = kwargs['component_template_name']
+            else:
+                page_template_definition = self.template_content.draft_template.definition
+                self.component_template_name = page_template_definition['contents'][self.content_key]['templateName']
+                
+        if not self.component_template_name:
+            raise ValueError('component_template_name is missing')
+
+
+    def get_initial(self):
+        # do not use super() because it would add contents of localized_template_content to initial
+        initial = {
+            'input_language': self.localized_template_content.language,
+        }
+        if self.component:
+            initial['uuid'] = self.component['uuid']
+        else:
+            initial['uuid'] = uuid.uuid4()
+        return initial
 
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
         context['content_key'] = self.content_key
         context['component'] = self.component
+        context['component_template_name'] = self.component_template_name
         return context
 
 
@@ -312,7 +370,8 @@ class ManageComponent(ManageTemplateContentCommon, AppMixin, WithLocalizedTempla
         if form_class is None:
             form_class = self.get_form_class()
 
-        return form_class(self.app, self.template_content, self.localized_template_content, self.content_key, self.component, **self.get_form_kwargs())
+        return form_class(self.app, self.template_content, self.localized_template_content, self.content_key, self.component_template_name,
+                          self.component, **self.get_form_kwargs())
 
 
     def form_valid(self, form):
@@ -320,12 +379,13 @@ class ManageComponent(ManageTemplateContentCommon, AppMixin, WithLocalizedTempla
         if not self.localized_template_content.draft_contents:
             self.localized_template_content.draft_contents = {}
 
-        template = self.template_content.get_component_template(self.content_key)
+        component_template = Template(self.app, self.component_template_name, 'component')
 
-        updated_component = self.get_updated_content_dict(template.definition, self.component, form)
+        updated_component = self.get_updated_content_dict(component_template.definition, self.component, form)
         if not form.cleaned_data['uuid']:
             raise ValueError('uuid is missing')
         updated_component['uuid'] = str(form.cleaned_data['uuid'])
+        updated_component['templateName'] = self.component_template_name
 
         self.localized_template_content.add_or_update_component(self.content_key, updated_component)
 
@@ -334,9 +394,9 @@ class ManageComponent(ManageTemplateContentCommon, AppMixin, WithLocalizedTempla
         context = self.get_context_data(**self.kwargs)
         context['success'] = True
         return self.render_to_response(context)
+        
 
-
-class DeleteComponent(TemplateView):
+class DeleteComponent(AppMixin, TemplateView):
 
     template_name = 'template_content/ajax/delete_component.html'
 
@@ -346,20 +406,31 @@ class DeleteComponent(TemplateView):
         return super().dispatch(request, *args, **kwargs)
 
     def set_component(self, **kwargs):
-        self.app = App.objects.get(uid=kwargs['app_uid'])
         self.localized_template_content = LocalizedTemplateContent.objects.get(pk=kwargs['localized_template_content_id'])
         
         self.content_key = kwargs['content_key']
         self.component_uuid = kwargs['component_uuid']
         
-        self.component_template = self.localized_template_content.template_content.get_component_template(
-            self.content_key)
-        self.component_template_name = self.component_template.definition['templateName']
+        self.component = self.localized_template_content.get_component(self.content_key, self.component_uuid)
         
+        if not self.component:
+            raise ValueError(f'Component {self.component_uuid} not found')
+        
+        self.component_template_name = self.component.get('templateName', None)
+        
+        if not self.component_template_name:
+        
+            component_template = self.localized_template_content.template_content.get_component_template(
+                self.content_key)
+            
+            if component_template.definition['type'] == 'stream':
+                raise ValueError('Failed to get component_template_name.')
+            else:
+                self.component_template_name = component_template.definition['templateName']
+                
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
-        context['app'] = self.app
         context['localized_template_content'] = self.localized_template_content
         context['content_key'] = self.content_key
         context['component_uuid'] = self.component_uuid
@@ -376,10 +447,50 @@ class DeleteComponent(TemplateView):
         return self.render_to_response(context)
 
 
+
+class StoreComponentOrder(AppMixin, TemplateView):
+    
+    @method_decorator(ajax_required)
+    def post(self, request, *args, **kwargs):
+
+        success = False
+
+        order = request.POST.get('order', None)
+
+        if order:
+            
+            order = json.loads(order)
+            
+            localized_template_content = LocalizedTemplateContent.objects.get(pk=kwargs['localized_template_content_id'])
+            content_key = kwargs['content_key']
+            
+            content_definition = localized_template_content.template_content.draft_template.definition['contents'][content_key]
+            
+            if not content_definition or content_definition['type'] != 'stream':
+                return JsonResponse({'success':False, 'error':'Content is not a stream'})
+
+            existing_components = localized_template_content.draft_contents.get(content_key, [])
+
+            ordered_components = []
+            for component_uuid in order:
+                
+                for component in existing_components:
+                    if component['uuid'] == component_uuid:
+                        ordered_components.append(component)
+                        break
+                    
+            # make sure nothing gets lost
+            if len(ordered_components) != len(existing_components):
+                return JsonResponse({'success':False, 'error':'Components got lost.'})
+            
+            localized_template_content.draft_contents[content_key] = ordered_components
+            localized_template_content.save()
+            success = True
+        
+        return JsonResponse({'success':success})
 '''
     use the same form / template as for the primary language
-    but display the primary language abov ethe input fields
-    display images, bu do not offer translations for images
+    but display the primary language above the input fields
 '''
 class TranslateTemplateContent(ManageTemplateContentCommon, AppMixin, FormView):
     
@@ -387,11 +498,14 @@ class TranslateTemplateContent(ManageTemplateContentCommon, AppMixin, FormView):
     form_class = TranslateTemplateContentForm
 
     def dispatch(self, request, *args, **kwargs):
+        self.set_template_content(**kwargs)
+        return super().dispatch(request, *args, **kwargs)
+    
+    def set_template_content(self, **kwargs):
         self.template_content = TemplateContent.objects.get(pk=kwargs['template_content_id'])
         self.language = kwargs['language']
         self.localized_template_content = self.template_content.get_locale(self.language)
-        return super().dispatch(request, *args, **kwargs)
-
+        
     
     def form_valid(self, form):
 
@@ -413,7 +527,6 @@ class TranslateTemplateContent(ManageTemplateContentCommon, AppMixin, FormView):
     for successful image deletions and uploads
     reloads all fields if field is multi
 '''
-from .forms import TemplateContentFormFieldManager
 class GetTemplateContentFormFields(FormView):
 
     template_name = 'template_content/ajax/reloaded_form_fields.html'
@@ -452,7 +565,7 @@ class GetTemplateContentFormFields(FormView):
 
 
 
-class ManageTemplateContentImage(ManageServerContentImage):
+class ManageTemplateContentImage(AppMixin, ManageServerContentImageWithText):
 
     template_name = 'template_content/ajax/manage_template_content_image.html'
 
@@ -476,7 +589,6 @@ class DeleteTemplateContentImage(DeleteServerContentImage):
         return context
 
 
-
 '''
     Create the component with its uuid
     Upload the image
@@ -493,16 +605,18 @@ class ContextFromComponentIdentifier:
     def set_component(self):
         image_type = self.get_image_type()
         content_identifiers = image_type.split(':')
+        
         self.content_key = content_identifiers[-1]
         self.component_key = content_identifiers[0]
         self.component_uuid = content_identifiers[1]
+            
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
         self.set_component()
-        context['content_key'] = self.content_key
-        context['component_key'] = self.component_key
-        context['component_uuid'] = self.component_uuid
+        context['content_key'] = self.content_key # the field in page.contents
+        context['component_key'] = self.component_key # page.contents[content_key][component_key] - the key that holds the image
+        context['component_uuid'] = self.component_uuid # the uuid of the component instance
         return context
 
 
@@ -513,17 +627,20 @@ class ManageComponentImage(ContextFromComponentIdentifier, ManageTemplateContent
     def save_image(self, form):
         self.set_component()
         # check if component exits, and save if if not
+        
         component = self.content_instance.get_component(self.component_key, self.component_uuid)
 
         if not component:
             new_component = {
                 'uuid': self.component_uuid
             }
+
+            # add component to page contents -> we need a component_uuid
             self.content_instance.add_or_update_component(self.component_key, new_component)
         super().save_image(form)
 
 
-class DeleteComponentImage(ContextFromComponentIdentifier, DeleteTemplateContentImage):
+class DeleteComponentImage(AppMixin, ContextFromComponentIdentifier, DeleteTemplateContentImage):
     template_name = 'template_content/ajax/delete_component_image.html'
 
     def get_image_type(self):
@@ -538,14 +655,15 @@ class PublishTemplateContent(AppMixin, TemplateView):
     template_name = 'template_content/template_content_list_entry.html'
 
     def dispatch(self, request, *args, **kwargs):
-
+        self.set_template_content(**kwargs)
+        return super().dispatch(request, *args, **kwargs)
+    
+    def set_template_content(self, **kwargs):
         self.template_content = TemplateContent.objects.get(pk=kwargs['template_content_id'])
         self.localized_template_content = self.template_content.get_locale(
             self.template_content.app.primary_language)
         self.language = kwargs.get('language', 'all')
 
-        return super().dispatch(request, *args, **kwargs)
-
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
         context['localized_template_content'] = self.localized_template_content
@@ -580,7 +698,7 @@ class UnpublishTemplateContent(AppMixin, TemplateView):
         return self.render_to_response(context)
 
 
-class DeleteTemplateContent(AjaxDeleteView):
+class DeleteTemplateContent(AppMixin, AjaxDeleteView):
     model = TemplateContent
     
 
@@ -675,7 +793,7 @@ class PublishNavigation(AppMixin, TemplateView):
         return context
 
 
-class DeleteNavigation(AjaxDeleteView):
+class DeleteNavigation(AppMixin, AjaxDeleteView):
     model = Navigation
 
 
@@ -787,5 +905,106 @@ class ManageNavigationEntry(AppMixin, FormLanguageMixin, FormView):
         return self.render_to_response(context)
 
 
-class DeleteNavigationEntry(AjaxDeleteView):
-    model = NavigationEntry
\ No newline at end of file
+class DeleteNavigationEntry(AppMixin, AjaxDeleteView):
+    model = NavigationEntry
+
+
+class ComponentContentView(ManageComponent):
+
+    template_name = 'template_content/ajax/component_content_view.html'
+
+    @method_decorator(ajax_required)
+    def dispatch(self, request, *args, **kwargs):
+        self.set_template_content(**kwargs)
+        self.set_component(**kwargs)
+        return super().dispatch(request, *args, **kwargs)
+
+    def set_template_content(self, **kwargs):
+        self.template_content = TemplateContent.objects.get(pk=kwargs['template_content_id'])
+        self.app = self.template_content.app
+        self.language = self.app.primary_language
+        self.localized_template_content = self.template_content.get_locale(self.language)
+
+
+class TranslateNavigation(AppMixin, FormView):
+    
+    template_name = 'template_content/translate_localized_navigation.html'
+    form_class = TranslateNavigationForm
+
+    def dispatch(self, request, *args, **kwargs):
+        self.set_navigation(**kwargs)
+        return super().dispatch(request, *args, **kwargs)
+    
+    def set_navigation(self, **kwargs):
+        self.app = App.objects.get(uid=kwargs['app_uid'])
+        self.navigation = Navigation.objects.get(pk=kwargs['pk'])
+        self.language = kwargs['language']
+        self.localized_navigation = self.navigation.get_locale(self.language)
+        self.primary_locale_navigation = self.navigation.get_locale(self.app.primary_language)
+
+
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context['localized_navigation'] = self.localized_navigation
+        context['navigation'] = self.navigation
+        context['primary_locale_navigation'] = self.primary_locale_navigation
+        context['language'] = self.language
+        context['saved'] = False
+        return context
+    
+
+    def get_form_kwargs(self):
+        form_kwargs = super().get_form_kwargs()
+        form_kwargs['language'] = self.language
+        return form_kwargs
+    
+    
+    def get_initial(self):
+        initial = super().get_initial()
+        if self.localized_navigation:
+            initial['name'] = self.localized_navigation.name
+        return initial
+
+
+    def get_form(self, form_class=None):
+        if form_class is None:
+            form_class = self.get_form_class()
+        return form_class(self.app, self.navigation, **self.get_form_kwargs())
+    
+
+    def save_localized_navigation(self, form):
+        
+        for field in form:
+
+            if hasattr(field.field, 'navigation_entry'):
+
+                navigation_entry = field.field.navigation_entry
+                localized_navigation_entry = navigation_entry.get_locale(self.language)
+
+                link_name = form.cleaned_data.get(field.name, '')
+
+                if not link_name or link_name == '':
+                    if localized_navigation_entry:
+                        localized_navigation_entry.delete()
+
+                else:
+                    if not localized_navigation_entry:
+                        localized_navigation_entry = LocalizedNavigationEntry(
+                            navigation_entry = navigation_entry,
+                            language = self.language,
+                        )
+                        
+                    localized_navigation_entry.link_name = link_name
+                    localized_navigation_entry.save()
+
+    def form_valid(self, form):
+
+        if not self.localized_navigation:
+            self.localized_navigation = LocalizedNavigation.objects.create(self.navigation, self.language,
+                                                                           form.cleaned_data['name'])
+
+        self.save_localized_navigation(form)
+
+        context = self.get_context_data(**self.kwargs)
+        context['saved'] = True
+        return self.render_to_response(context)
\ No newline at end of file
